@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*                                                                        *)
 (*  Copyright 2011 Jun Furuse                                             *)
-(*  Copyright 2012,2013 OCamlPro                                          *)
+(*  Copyright 2012,2015 OCamlPro                                          *)
 (*                                                                        *)
 (*  All rights reserved.This file is distributed under the terms of the   *)
 (*  GNU Lesser General Public License version 3.0 with linking            *)
@@ -283,6 +283,8 @@ let rec close_top_let = function
 let rec unwind f path = match path with
   | { kind } :: _ when f kind -> path
   | { kind=KCodeInComment } :: _ -> path
+  | { kind = KAttrId _ } :: { kind } :: _ when f kind -> path
+  (* never remove the KattrId following a KExtendedItem *)
   | _ :: path -> unwind f path
   | [] -> []
 
@@ -544,21 +546,14 @@ let rec update_path config block stream tok =
   in
   (* Add a new child block *)
   let append kind pos ?(pad=config.i_base) = function
-    | ({kind = KAttrId (_, _) } as h1) ::
-      (({kind = KExtendedExpr [] } as h2)  :: _ as path) ->
-        (* 'KAttrId' emulates an opening brace right before the current
-           'tok' (i.e. the first inner-expr token ) *)
-        let path =
-          let indent =
-            if starts_line then h2.indent else block.toff + tok.offset in
-          let pad = if starts_line then config.i_base else 0 in
-          {h1 with indent; column=indent; pad } :: path in
-        node false kind pos pad path :: path
     | {kind = KAttrId (names, _)} ::
-      ({kind = KExtendedItem [] } as n) :: path ->
-        let path =
-          {n with kind = KExtendedItem (List.rev names);
-                  pad = config.i_ppx_stritem_ext } :: path in
+      ({kind = KExtendedItem [] | KExtendedExpr [] } as n) :: path ->
+        let n = { n with kind = match n.kind with
+            | KExtendedItem [] -> KExtendedItem (List.rev names)
+            | KExtendedExpr [] -> KExtendedExpr (List.rev names)
+            | _ -> assert false
+          } in
+        let path = {n with pad = config.i_ppx_stritem_ext } :: path in
         node false kind pos pad path :: path
     | path ->
         node false kind pos pad path :: path
@@ -728,11 +723,16 @@ let rec update_path config block stream tok =
   in
   let make_infix tok path =
     let op_prio, align, indent = op_prio_align_indent config tok.token in
+    let in_record =
+        match unwind_while (fun kind -> prio kind >= op_prio) path with
+        | Some ({ kind = KExpr _ } :: { kind = KBrace } :: _) -> true
+        | _ -> false in
     (* special cases *)
     let indent =
       (* don't back-indent operators when alone on their line
          (except BAR because that would disrupt typing) *)
       if indent < 0 && tok.token <> BAR
+         && not (tok.token = SEMI && in_record)
          && next_offset tok stream = None
       then 0 else indent
     in
@@ -813,9 +813,15 @@ let rec update_path config block stream tok =
       let path = before_append_atom block.path in
       append ~pad:4 (KExtendedExpr []) L path
   | LBRACKETATAT ->
-      append ~pad:4 (KExtendedItem []) L
+      let path =
         (unwind (function KBody k | k -> top_kind k || stritem_kind k)
-            block.path)
+             block.path)
+      in
+      let path = match path with
+        | {kind = KBody k | k} :: p -> if top_kind k then path else p
+        | [] -> []
+      in
+      append ~pad:4 (KExtendedItem []) L path
   | LBRACKETPERCENTPERCENT | LBRACKETATATAT ->
       append ~pad:4 (KExtendedItem []) L (unwind_top block.path)
   | LBRACKETBAR -> open_paren KBracketBar block.path
@@ -1135,7 +1141,7 @@ let rec update_path config block stream tok =
                 | Some (MATCH|TRY|FUN|FUNCTION) -> 0
                 | _ -> config.i_base
             in
-            append (KArrow m) L ~pad path
+            append (KArrow m) L ~pad (reset_line_indent config line path)
         | {kind=KWith m | KBar m} :: _ ->
             let pad =
               config.i_match_clause
@@ -1468,11 +1474,12 @@ let rec update_path config block stream tok =
                  match align_bar with
                  | Some indent ->
                      append (KComment (tok,indent)) (A indent) ~pad block.path
-                 | None ->  (* recursive call to indent like next line *)
+                 | None -> (* recursive call to indent like next line *)
                      let path = match next with
                        | Some ({token = EOF }, _) | None -> []
                        | Some (next,stream) ->
-                           update_path config block stream next
+                           update_path config block stream
+                             { next with newlines = tok.newlines }
                      in
                      let col = Path.indent path in
                      append (KComment (tok,col)) (A col) ~pad block.path)
