@@ -1,4 +1,3 @@
-
 val f
   :  int
   -> int
@@ -18,11 +17,6 @@ let _ =
     y
   else z
 
-let _ =
-  if x then y else
-  if x then y else
-    x
-
 type t
   =  int
   -> int
