@@ -1,21 +1,21 @@
 let () =
   f
     x
-    <:sexp_of<int>>
+    [%sexp_of int]
     y
 ;;
 
-(* yminsky *)
+(* y *)
 let z =
   some_function
-    <:sexp_of<foo>>
+    [%sexp_of foo]
 ;;
 
 let z =
   some_function
     argument
 
-let dwang =
+let d =
   print_sexp
-    <:sexp_of<unit>>
+    [%sexp_of unit]
     ()
